# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

from google.type import date_pb2  # type: ignore
from google.type import latlng_pb2  # type: ignore
from google.type import money_pb2  # type: ignore
import proto  # type: ignore

__protobuf__ = proto.module(
    package="google.maps.solar.v1",
    manifest={
        "DataLayerView",
        "ImageryQuality",
        "SolarPanelOrientation",
        "FindClosestBuildingInsightsRequest",
        "LatLngBox",
        "BuildingInsights",
        "SolarPotential",
        "RoofSegmentSizeAndSunshineStats",
        "SizeAndSunshineStats",
        "SolarPanel",
        "SolarPanelConfig",
        "RoofSegmentSummary",
        "FinancialAnalysis",
        "FinancialDetails",
        "SavingsOverTime",
        "LeasingSavings",
        "CashPurchaseSavings",
        "FinancedPurchaseSavings",
        "GetDataLayersRequest",
        "DataLayers",
        "GetGeoTiffRequest",
    },
)


class DataLayerView(proto.Enum):
    r"""What subset of the solar information to return.

    Values:
        DATA_LAYER_VIEW_UNSPECIFIED (0):
            Equivalent to FULL.
        DSM_LAYER (1):
            Get the DSM only.
        IMAGERY_LAYERS (2):
            Get the DSM, RGB, and mask.
        IMAGERY_AND_ANNUAL_FLUX_LAYERS (3):
            Get the DSM, RGB, mask, and annual flux.
        IMAGERY_AND_ALL_FLUX_LAYERS (4):
            Get the DSM, RGB, mask, annual flux, and
            monthly flux.
        FULL_LAYERS (5):
            Get all data.
    """
    DATA_LAYER_VIEW_UNSPECIFIED = 0
    DSM_LAYER = 1
    IMAGERY_LAYERS = 2
    IMAGERY_AND_ANNUAL_FLUX_LAYERS = 3
    IMAGERY_AND_ALL_FLUX_LAYERS = 4
    FULL_LAYERS = 5


class ImageryQuality(proto.Enum):
    r"""The quality of the imagery used to compute some API result.

    Note: Regardless of imagery quality level, DSM outputs always
    have a resolution of 0.1 m/pixel, monthly flux outputs always
    have a resolution of 0.5 m/pixel, and hourly shade outputs
    always have a resolution of 1 m/pixel.

    Values:
        IMAGERY_QUALITY_UNSPECIFIED (0):
            No quality is known.
        HIGH (1):
            The underlying imagery and DSM data were
            processed at 0.1 m/pixel.
        MEDIUM (2):
            The underlying imagery and DSM data were
            processed at 0.25 m/pixel.
        LOW (3):
            The underlying imagery and DSM data were
            processed at 0.5 m/pixel.
    """
    IMAGERY_QUALITY_UNSPECIFIED = 0
    HIGH = 1
    MEDIUM = 2
    LOW = 3


class SolarPanelOrientation(proto.Enum):
    r"""The orientation of a solar panel. This must be interpreted
    relative to the azimuth of the roof segment that the panel is
    placed on.

    Values:
        SOLAR_PANEL_ORIENTATION_UNSPECIFIED (0):
            No panel orientation is known.
        LANDSCAPE (1):
            A ``LANDSCAPE`` panel has its long edge perpendicular to the
            azimuth direction of the roof segment that it is placed on.
        PORTRAIT (2):
            A ``PORTRAIT`` panel has its long edge parallel to the
            azimuth direction of the roof segment that it is placed on.
    """
    SOLAR_PANEL_ORIENTATION_UNSPECIFIED = 0
    LANDSCAPE = 1
    PORTRAIT = 2


class FindClosestBuildingInsightsRequest(proto.Message):
    r"""Request message for ``Solar.FindClosestBuildingInsights``.

    Attributes:
        location (google.type.latlng_pb2.LatLng):
            Required. The longitude and latitude from
            which the API looks for the nearest known
            building.
        required_quality (google.maps.solar_v1.types.ImageryQuality):
            Optional. The minimum quality level allowed
            in the results. No result with lower quality
            than this will be returned. Not specifying this
            is equivalent to restricting to HIGH quality
            only.
        exact_quality_required (bool):
            Optional. Whether to require exact quality of the imagery.
            If set to false, the ``required_quality`` field is
            interpreted as the minimum required quality, such that HIGH
            quality imagery may be returned when ``required_quality`` is
            set to MEDIUM. If set to true, ``required_quality`` is
            interpreted as the exact required quality and only
            ``MEDIUM`` quality imagery is returned if
            ``required_quality`` is set to ``MEDIUM``.
    """

    location: latlng_pb2.LatLng = proto.Field(
        proto.MESSAGE,
        number=1,
        message=latlng_pb2.LatLng,
    )
    required_quality: "ImageryQuality" = proto.Field(
        proto.ENUM,
        number=3,
        enum="ImageryQuality",
    )
    exact_quality_required: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class LatLngBox(proto.Message):
    r"""A bounding box in lat/lng coordinates.

    Attributes:
        sw (google.type.latlng_pb2.LatLng):
            The southwest corner of the box.
        ne (google.type.latlng_pb2.LatLng):
            The northeast corner of the box.
    """

    sw: latlng_pb2.LatLng = proto.Field(
        proto.MESSAGE,
        number=1,
        message=latlng_pb2.LatLng,
    )
    ne: latlng_pb2.LatLng = proto.Field(
        proto.MESSAGE,
        number=2,
        message=latlng_pb2.LatLng,
    )


class BuildingInsights(proto.Message):
    r"""Response message for ``Solar.FindClosestBuildingInsights``.
    Information about the location, dimensions, and solar potential of a
    building.

    Attributes:
        name (str):
            The resource name for the building, of the format
            ``building/<place ID>``.
        center (google.type.latlng_pb2.LatLng):
            A point near the center of the building.
        bounding_box (google.maps.solar_v1.types.LatLngBox):
            The bounding box of the building.
        imagery_date (google.type.date_pb2.Date):
            Date that the underlying imagery was
            acquired. This is approximate.
        imagery_processed_date (google.type.date_pb2.Date):
            When processing was completed on this
            imagery.
        postal_code (str):
            Postal code (e.g., US zip code) this building
            is contained by.
        administrative_area (str):
            Administrative area 1 (e.g., in the US, the
            state) that contains this building. For example,
            in the US, the abbreviation might be "MA" or
            "CA.".
        statistical_area (str):
            Statistical area (e.g., US census tract) this
            building is in.
        region_code (str):
            Region code for the country (or region) this
            building is in.
        solar_potential (google.maps.solar_v1.types.SolarPotential):
            Solar potential of the building.
        imagery_quality (google.maps.solar_v1.types.ImageryQuality):
            The quality of the imagery used to compute
            the data for this building.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    center: latlng_pb2.LatLng = proto.Field(
        proto.MESSAGE,
        number=2,
        message=latlng_pb2.LatLng,
    )
    bounding_box: "LatLngBox" = proto.Field(
        proto.MESSAGE,
        number=9,
        message="LatLngBox",
    )
    imagery_date: date_pb2.Date = proto.Field(
        proto.MESSAGE,
        number=3,
        message=date_pb2.Date,
    )
    imagery_processed_date: date_pb2.Date = proto.Field(
        proto.MESSAGE,
        number=11,
        message=date_pb2.Date,
    )
    postal_code: str = proto.Field(
        proto.STRING,
        number=4,
    )
    administrative_area: str = proto.Field(
        proto.STRING,
        number=5,
    )
    statistical_area: str = proto.Field(
        proto.STRING,
        number=6,
    )
    region_code: str = proto.Field(
        proto.STRING,
        number=7,
    )
    solar_potential: "SolarPotential" = proto.Field(
        proto.MESSAGE,
        number=8,
        message="SolarPotential",
    )
    imagery_quality: "ImageryQuality" = proto.Field(
        proto.ENUM,
        number=10,
        enum="ImageryQuality",
    )


class SolarPotential(proto.Message):
    r"""Information about the solar potential of a building. A number of
    fields in this are defined in terms of "panels". The fields
    [panel_capacity_watts]
    [google.maps.solar.v1.SolarPotential.panel_capacity_watts],
    [panel_height_meters]
    [google.maps.solar.v1.SolarPotential.panel_height_meters], and
    [panel_width_meters]
    [google.maps.solar.v1.SolarPotential.panel_width_meters] describe
    the parameters of the model of panel used in these calculations.

    Attributes:
        max_array_panels_count (int):
            Size of the maximum array - that is, the
            maximum number of panels that can fit on the
            roof.
        panel_capacity_watts (float):
            Capacity, in watts, of the panel used in the
            calculations.
        panel_height_meters (float):
            Height, in meters in portrait orientation, of
            the panel used in the calculations.
        panel_width_meters (float):
            Width, in meters in portrait orientation, of
            the panel used in the calculations.
        panel_lifetime_years (int):
            The expected lifetime, in years, of the solar
            panels. This is used in the financial
            calculations.
        max_array_area_meters2 (float):
            Size, in square meters, of the maximum array.
        max_sunshine_hours_per_year (float):
            Maximum number of sunshine hours received per
            year, by any point on the roof. Sunshine hours
            are a measure of the total amount of insolation
            (energy) received per year. 1 sunshine hour = 1
            kWh per kW (where kW refers to kW of capacity
            under Standard Testing Conditions).
        carbon_offset_factor_kg_per_mwh (float):
            Equivalent amount of CO2 produced per MWh of
            grid electricity. This is a measure of the
            carbon intensity of grid electricity displaced
            by solar electricity.
        whole_roof_stats (google.maps.solar_v1.types.SizeAndSunshineStats):
            Total size and sunlight quantiles for the part of the roof
            that was assigned to some roof segment. Despite the name,
            this may not include the entire building. See
            [building_stats]
            [google.maps.solar.v1.SolarPotential.building_stats].
        building_stats (google.maps.solar_v1.types.SizeAndSunshineStats):
            Size and sunlight quantiles for the entire building,
            including parts of the roof that were not assigned to some
            roof segment. Because the orientations of these parts are
            not well characterised, the roof area estimate is
            unreliable, but the ground area estimate is reliable. It may
            be that a more reliable whole building roof area can be
            obtained by scaling the roof area from [whole_roof_stats]
            [google.maps.solar.v1.SolarPotential.whole_roof_stats] by
            the ratio of the ground areas of ``building_stats`` and
            ``whole_roof_stats``.
        roof_segment_stats (MutableSequence[google.maps.solar_v1.types.RoofSegmentSizeAndSunshineStats]):
            Size and sunlight quantiles for each roof
            segment.
        solar_panels (MutableSequence[google.maps.solar_v1.types.SolarPanel]):
            Each [SolarPanel] [google.maps.solar.v1.SolarPanel]
            describes a single solar panel. They are listed in the order
            that the panel layout algorithm placed this. This is
            usually, though not always, in decreasing order of annual
            energy production.
        solar_panel_configs (MutableSequence[google.maps.solar_v1.types.SolarPanelConfig]):
            Each [SolarPanelConfig]
            [google.maps.solar.v1.SolarPanelConfig] describes a
            different arrangement of solar panels on the roof. They are
            in order of increasing number of panels. The
            ``SolarPanelConfig`` with [panels_count]
            [google.maps.solar.v1.SolarPanelConfig.panels_count]=N is
            based on the first N panels in the ``solar_panels`` list.
            This field is only populated if at least 4 panels can fit on
            a roof.
        financial_analyses (MutableSequence[google.maps.solar_v1.types.FinancialAnalysis]):
            A [FinancialAnalysis]
            [google.maps.solar.v1.FinancialAnalysis] gives the savings
            from going solar assuming a given monthly bill and a given
            electricity provider. They are in order of increasing order
            of monthly bill amount. This field will be empty for
            buildings in areas for which the Solar API does not have
            enough information to perform financial computations.
    """

    max_array_panels_count: int = proto.Field(
        proto.INT32,
        number=1,
    )
    panel_capacity_watts: float = proto.Field(
        proto.FLOAT,
        number=9,
    )
    panel_height_meters: float = proto.Field(
        proto.FLOAT,
        number=10,
    )
    panel_width_meters: float = proto.Field(
        proto.FLOAT,
        number=11,
    )
    panel_lifetime_years: int = proto.Field(
        proto.INT32,
        number=12,
    )
    max_array_area_meters2: float = proto.Field(
        proto.FLOAT,
        number=2,
    )
    max_sunshine_hours_per_year: float = proto.Field(
        proto.FLOAT,
        number=3,
    )
    carbon_offset_factor_kg_per_mwh: float = proto.Field(
        proto.FLOAT,
        number=4,
    )
    whole_roof_stats: "SizeAndSunshineStats" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="SizeAndSunshineStats",
    )
    building_stats: "SizeAndSunshineStats" = proto.Field(
        proto.MESSAGE,
        number=13,
        message="SizeAndSunshineStats",
    )
    roof_segment_stats: MutableSequence[
        "RoofSegmentSizeAndSunshineStats"
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=6,
        message="RoofSegmentSizeAndSunshineStats",
    )
    solar_panels: MutableSequence["SolarPanel"] = proto.RepeatedField(
        proto.MESSAGE,
        number=14,
        message="SolarPanel",
    )
    solar_panel_configs: MutableSequence["SolarPanelConfig"] = proto.RepeatedField(
        proto.MESSAGE,
        number=7,
        message="SolarPanelConfig",
    )
    financial_analyses: MutableSequence["FinancialAnalysis"] = proto.RepeatedField(
        proto.MESSAGE,
        number=8,
        message="FinancialAnalysis",
    )


class RoofSegmentSizeAndSunshineStats(proto.Message):
    r"""Information about the size and sunniness quantiles of a roof
    segment.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        pitch_degrees (float):
            Angle of the roof segment relative to the
            theoretical ground plane. 0 = parallel to the
            ground, 90 = perpendicular to the ground.

            This field is a member of `oneof`_ ``_pitch_degrees``.
        azimuth_degrees (float):
            Compass direction the roof segment is pointing in. 0 =
            North, 90 = East, 180 = South. For a "flat" roof segment
            (``pitch_degrees`` very near 0), azimuth is not well
            defined, so for consistency, we define it arbitrarily to be
            0 (North).

            This field is a member of `oneof`_ ``_azimuth_degrees``.
        stats (google.maps.solar_v1.types.SizeAndSunshineStats):
            Total size and sunlight quantiles for the
            roof segment.
        center (google.type.latlng_pb2.LatLng):
            A point near the center of the roof segment.
        bounding_box (google.maps.solar_v1.types.LatLngBox):
            The bounding box of the roof segment.
        plane_height_at_center_meters (float):
            The height of the roof segment plane, in meters above sea
            level, at the point designated by ``center``. Together with
            the pitch, azimuth, and center location, this fully defines
            the roof segment plane.

            This field is a member of `oneof`_ ``_plane_height_at_center_meters``.
    """

    pitch_degrees: float = proto.Field(
        proto.FLOAT,
        number=1,
        optional=True,
    )
    azimuth_degrees: float = proto.Field(
        proto.FLOAT,
        number=2,
        optional=True,
    )
    stats: "SizeAndSunshineStats" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="SizeAndSunshineStats",
    )
    center: latlng_pb2.LatLng = proto.Field(
        proto.MESSAGE,
        number=4,
        message=latlng_pb2.LatLng,
    )
    bounding_box: "LatLngBox" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="LatLngBox",
    )
    plane_height_at_center_meters: float = proto.Field(
        proto.FLOAT,
        number=6,
        optional=True,
    )


class SizeAndSunshineStats(proto.Message):
    r"""Size and sunniness quantiles of a roof, or part of a roof.

    Attributes:
        area_meters2 (float):
            The area of the roof or roof segment, in m^2.
            This is the roof area (accounting for tilt), not
            the ground footprint area.
        sunshine_quantiles (MutableSequence[float]):
            Quantiles of the pointwise sunniness across the area. If
            there are N values here, this represents the (N-1)-iles. For
            example, if there are 5 values, then they would be the
            quartiles (min, 25%, 50%, 75%, max). Values are in annual
            kWh/kW like [max_sunshine_hours_per_year]
            [google.maps.solar.v1.SolarPotential.max_sunshine_hours_per_year].
        ground_area_meters2 (float):
            The ground footprint area covered by the roof
            or roof segment, in m^2.
    """

    area_meters2: float = proto.Field(
        proto.FLOAT,
        number=1,
    )
    sunshine_quantiles: MutableSequence[float] = proto.RepeatedField(
        proto.FLOAT,
        number=2,
    )
    ground_area_meters2: float = proto.Field(
        proto.FLOAT,
        number=3,
    )


class SolarPanel(proto.Message):
    r"""SolarPanel describes the position, orientation, and production of a
    single solar panel. See the [panel_height_meters]
    [google.maps.solar.v1.SolarPotential.panel_height_meters],
    [panel_width_meters]
    [google.maps.solar.v1.SolarPotential.panel_width_meters], and
    [panel_capacity_watts]
    [google.maps.solar.v1.SolarPotential.panel_capacity_watts] fields in
    [SolarPotential] [google.maps.solar.v1.SolarPotential] for
    information on the parameters of the panel.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        center (google.type.latlng_pb2.LatLng):
            The centre of the panel.
        orientation (google.maps.solar_v1.types.SolarPanelOrientation):
            The orientation of the panel.
        yearly_energy_dc_kwh (float):
            How much sunlight energy this layout captures
            over the course of a year, in DC kWh.
        segment_index (int):
            Index in [roof_segment_stats]
            [google.maps.solar.v1.SolarPotential.roof_segment_stats] of
            the ``RoofSegmentSizeAndSunshineStats`` which corresponds to
            the roof segment that this panel is placed on.

            This field is a member of `oneof`_ ``_segment_index``.
    """

    center: latlng_pb2.LatLng = proto.Field(
        proto.MESSAGE,
        number=1,
        message=latlng_pb2.LatLng,
    )
    orientation: "SolarPanelOrientation" = proto.Field(
        proto.ENUM,
        number=2,
        enum="SolarPanelOrientation",
    )
    yearly_energy_dc_kwh: float = proto.Field(
        proto.FLOAT,
        number=3,
    )
    segment_index: int = proto.Field(
        proto.INT32,
        number=4,
        optional=True,
    )


class SolarPanelConfig(proto.Message):
    r"""SolarPanelConfig describes a particular placement of solar
    panels on the roof.

    Attributes:
        panels_count (int):
            Total number of panels. Note that this is redundant to (the
            sum of) the corresponding fields in [roof_segment_summaries]
            [google.maps.solar.v1.SolarPanelConfig.roof_segment_summaries].
        yearly_energy_dc_kwh (float):
            How much sunlight energy this layout captures
            over the course of a year, in DC kWh, assuming
            the panels described above.
        roof_segment_summaries (MutableSequence[google.maps.solar_v1.types.RoofSegmentSummary]):
            Information about the production of each roof segment that
            is carrying at least one panel in this layout.
            ``roof_segment_summaries[i]`` describes the i-th roof
            segment, including its size, expected production and
            orientation.
    """

    panels_count: int = proto.Field(
        proto.INT32,
        number=1,
    )
    yearly_energy_dc_kwh: float = proto.Field(
        proto.FLOAT,
        number=2,
    )
    roof_segment_summaries: MutableSequence["RoofSegmentSummary"] = proto.RepeatedField(
        proto.MESSAGE,
        number=4,
        message="RoofSegmentSummary",
    )


class RoofSegmentSummary(proto.Message):
    r"""Information about a roof segment on the building, with some
    number of panels placed on it.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        pitch_degrees (float):
            Angle of the roof segment relative to the
            theoretical ground plane. 0 = parallel to the
            ground, 90 = perpendicular to the ground.

            This field is a member of `oneof`_ ``_pitch_degrees``.
        azimuth_degrees (float):
            Compass direction the roof segment is pointing in. 0 =
            North, 90 = East, 180 = South. For a "flat" roof segment
            (``pitch_degrees`` very near 0), azimuth is not well
            defined, so for consistency, we define it arbitrarily to be
            0 (North).

            This field is a member of `oneof`_ ``_azimuth_degrees``.
        panels_count (int):
            The total number of panels on this segment.
        yearly_energy_dc_kwh (float):
            How much sunlight energy this part of the
            layout captures over the course of a year, in DC
            kWh, assuming the panels described above.
        segment_index (int):
            Index in [roof_segment_stats]
            [google.maps.solar.v1.SolarPotential.roof_segment_stats] of
            the corresponding ``RoofSegmentSizeAndSunshineStats``.

            This field is a member of `oneof`_ ``_segment_index``.
    """

    pitch_degrees: float = proto.Field(
        proto.FLOAT,
        number=2,
        optional=True,
    )
    azimuth_degrees: float = proto.Field(
        proto.FLOAT,
        number=3,
        optional=True,
    )
    panels_count: int = proto.Field(
        proto.INT32,
        number=7,
    )
    yearly_energy_dc_kwh: float = proto.Field(
        proto.FLOAT,
        number=8,
    )
    segment_index: int = proto.Field(
        proto.INT32,
        number=9,
        optional=True,
    )


class FinancialAnalysis(proto.Message):
    r"""Analysis of the cost and benefits of the optimum solar layout
    for a particular electric bill size.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        monthly_bill (google.type.money_pb2.Money):
            The monthly electric bill this analysis
            assumes.
        default_bill (bool):
            Whether this is the bill size selected to be the default
            bill for the area this building is in. Exactly one
            ``FinancialAnalysis`` in ``BuildingSolarPotential`` should
            have ``default_bill`` set.
        average_kwh_per_month (float):
            How much electricity the house uses in an
            average month, based on the bill size and the
            local electricity rates.
        panel_config_index (int):
            Index in [solar_panel_configs]
            [google.maps.solar.v1.SolarPotential.solar_panel_configs] of
            the optimum solar layout for this bill size. This can be -1
            indicating that there is no layout. In this case, the
            remaining submessages will be omitted.

            This field is a member of `oneof`_ ``_panel_config_index``.
        financial_details (google.maps.solar_v1.types.FinancialDetails):
            Financial information that applies regardless
            of the financing method used.
        leasing_savings (google.maps.solar_v1.types.LeasingSavings):
            Cost and benefit of leasing the solar panels.
        cash_purchase_savings (google.maps.solar_v1.types.CashPurchaseSavings):
            Cost and benefit of buying the solar panels
            with cash.
        financed_purchase_savings (google.maps.solar_v1.types.FinancedPurchaseSavings):
            Cost and benefit of buying the solar panels
            by financing the purchase.
    """

    monthly_bill: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=3,
        message=money_pb2.Money,
    )
    default_bill: bool = proto.Field(
        proto.BOOL,
        number=4,
    )
    average_kwh_per_month: float = proto.Field(
        proto.FLOAT,
        number=5,
    )
    panel_config_index: int = proto.Field(
        proto.INT32,
        number=6,
        optional=True,
    )
    financial_details: "FinancialDetails" = proto.Field(
        proto.MESSAGE,
        number=7,
        message="FinancialDetails",
    )
    leasing_savings: "LeasingSavings" = proto.Field(
        proto.MESSAGE,
        number=8,
        message="LeasingSavings",
    )
    cash_purchase_savings: "CashPurchaseSavings" = proto.Field(
        proto.MESSAGE,
        number=9,
        message="CashPurchaseSavings",
    )
    financed_purchase_savings: "FinancedPurchaseSavings" = proto.Field(
        proto.MESSAGE,
        number=10,
        message="FinancedPurchaseSavings",
    )


class FinancialDetails(proto.Message):
    r"""Details of a financial analysis. Some of these details are already
    stored at higher levels (e.g., out of pocket cost). Total money
    amounts are over a lifetime period defined by the
    [panel_lifetime_years]
    [google.maps.solar.v1.SolarPotential.panel_lifetime_years] field in
    [SolarPotential] [google.maps.solar.v1.SolarPotential]. Note: The
    out of pocket cost of purchasing the panels is given in the
    [out_of_pocket_cost]
    [google.maps.solar.v1.CashPurchaseSavings.out_of_pocket_cost] field
    in [CashPurchaseSavings] [google.maps.solar.v1.CashPurchaseSavings].


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        initial_ac_kwh_per_year (float):
            How many AC kWh we think the solar panels
            will generate in their first year.
        remaining_lifetime_utility_bill (google.type.money_pb2.Money):
            Utility bill for electricity not produced by
            solar, for the lifetime of the panels.
        federal_incentive (google.type.money_pb2.Money):
            Amount of money available from federal
            incentives; this applies if the user buys (with
            or without a loan) the panels.
        state_incentive (google.type.money_pb2.Money):
            Amount of money available from state
            incentives; this applies if the user buys (with
            or without a loan) the panels.
        utility_incentive (google.type.money_pb2.Money):
            Amount of money available from utility
            incentives; this applies if the user buys (with
            or without a loan) the panels.
        lifetime_srec_total (google.type.money_pb2.Money):
            Amount of money the user will receive from
            Solar Renewable Energy Credits over the panel
            lifetime; this applies if the user buys (with or
            without a loan) the panels.
        cost_of_electricity_without_solar (google.type.money_pb2.Money):
            Total cost of electricity the user would have
            paid over the lifetime period if they didn't
            install solar.
        net_metering_allowed (bool):
            Whether net metering is allowed.
        solar_percentage (float):
            Percentage (0-100) of the user's power
            supplied by solar. Valid for the first year but
            approximately correct for future years.

            This field is a member of `oneof`_ ``_solar_percentage``.
        percentage_exported_to_grid (float):
            The percentage (0-100) of solar electricity
            production we assumed was exported to the grid,
            based on the first quarter of production. This
            affects the calculations if net metering is not
            allowed.

            This field is a member of `oneof`_ ``_percentage_exported_to_grid``.
    """

    initial_ac_kwh_per_year: float = proto.Field(
        proto.FLOAT,
        number=1,
    )
    remaining_lifetime_utility_bill: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=2,
        message=money_pb2.Money,
    )
    federal_incentive: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=3,
        message=money_pb2.Money,
    )
    state_incentive: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=4,
        message=money_pb2.Money,
    )
    utility_incentive: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=5,
        message=money_pb2.Money,
    )
    lifetime_srec_total: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=6,
        message=money_pb2.Money,
    )
    cost_of_electricity_without_solar: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=7,
        message=money_pb2.Money,
    )
    net_metering_allowed: bool = proto.Field(
        proto.BOOL,
        number=8,
    )
    solar_percentage: float = proto.Field(
        proto.FLOAT,
        number=9,
        optional=True,
    )
    percentage_exported_to_grid: float = proto.Field(
        proto.FLOAT,
        number=10,
        optional=True,
    )


class SavingsOverTime(proto.Message):
    r"""Financial information that's shared between different
    financing methods.

    Attributes:
        savings_year1 (google.type.money_pb2.Money):
            Savings in the first year after panel
            installation.
        savings_year20 (google.type.money_pb2.Money):
            Savings in the first twenty years after panel
            installation.
        present_value_of_savings_year20 (google.type.money_pb2.Money):
            Using the assumed discount rate, what is the
            present value of the cumulative 20-year savings?
        savings_lifetime (google.type.money_pb2.Money):
            Savings in the entire panel lifetime.
        present_value_of_savings_lifetime (google.type.money_pb2.Money):
            Using the assumed discount rate, what is the
            present value of the cumulative lifetime
            savings?
        financially_viable (bool):
            Indicates whether this scenario is
            financially viable.  Will be false for scenarios
            with poor financial viability (e.g.,
            money-losing).
    """

    savings_year1: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=1,
        message=money_pb2.Money,
    )
    savings_year20: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=2,
        message=money_pb2.Money,
    )
    present_value_of_savings_year20: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=3,
        message=money_pb2.Money,
    )
    savings_lifetime: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=5,
        message=money_pb2.Money,
    )
    present_value_of_savings_lifetime: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=6,
        message=money_pb2.Money,
    )
    financially_viable: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class LeasingSavings(proto.Message):
    r"""Cost and benefit of leasing a particular configuration of
    solar panels with a particular electricity usage.

    Attributes:
        leases_allowed (bool):
            Whether leases are allowed in this
            juristiction (leases are not allowed in some
            states). If this field is false, then the values
            in this message should probably be ignored.
        leases_supported (bool):
            Whether leases are supported in this juristiction by the
            financial calculation engine. If this field is false, then
            the values in this message should probably be ignored. This
            is independent of ``leases_allowed``: in some areas leases
            are allowed, but under conditions that aren't handled by the
            financial models.
        annual_leasing_cost (google.type.money_pb2.Money):
            Estimated annual leasing cost.
        savings (google.maps.solar_v1.types.SavingsOverTime):
            How much is saved (or not) over the lifetime
            period.
    """

    leases_allowed: bool = proto.Field(
        proto.BOOL,
        number=1,
    )
    leases_supported: bool = proto.Field(
        proto.BOOL,
        number=2,
    )
    annual_leasing_cost: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=3,
        message=money_pb2.Money,
    )
    savings: "SavingsOverTime" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="SavingsOverTime",
    )


class CashPurchaseSavings(proto.Message):
    r"""Cost and benefit of an outright purchase of a particular
    configuration of solar panels with a particular electricity
    usage.


    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        out_of_pocket_cost (google.type.money_pb2.Money):
            Initial cost before tax incentives: the amount that must be
            paid out-of-pocket. Contrast with ``upfront_cost``, which is
            after tax incentives.
        upfront_cost (google.type.money_pb2.Money):
            Initial cost after tax incentives: it's the amount that must
            be paid during first year. Contrast with
            ``out_of_pocket_cost``, which is before tax incentives.
        rebate_value (google.type.money_pb2.Money):
            The value of all tax rebates.
        payback_years (float):
            Number of years until payback occurs. A
            negative value means payback never occurs within
            the lifetime period.

            This field is a member of `oneof`_ ``_payback_years``.
        savings (google.maps.solar_v1.types.SavingsOverTime):
            How much is saved (or not) over the lifetime
            period.
    """

    out_of_pocket_cost: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=1,
        message=money_pb2.Money,
    )
    upfront_cost: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=2,
        message=money_pb2.Money,
    )
    rebate_value: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=3,
        message=money_pb2.Money,
    )
    payback_years: float = proto.Field(
        proto.FLOAT,
        number=4,
        optional=True,
    )
    savings: "SavingsOverTime" = proto.Field(
        proto.MESSAGE,
        number=5,
        message="SavingsOverTime",
    )


class FinancedPurchaseSavings(proto.Message):
    r"""Cost and benefit of using a loan to buy a particular
    configuration of solar panels with a particular electricity
    usage.

    Attributes:
        annual_loan_payment (google.type.money_pb2.Money):
            Annual loan payments.
        rebate_value (google.type.money_pb2.Money):
            The value of all tax rebates (including
            Federal Investment Tax Credit (ITC)).
        loan_interest_rate (float):
            The interest rate on loans assumed in this
            set of calculations.
        savings (google.maps.solar_v1.types.SavingsOverTime):
            How much is saved (or not) over the lifetime
            period.
    """

    annual_loan_payment: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=1,
        message=money_pb2.Money,
    )
    rebate_value: money_pb2.Money = proto.Field(
        proto.MESSAGE,
        number=2,
        message=money_pb2.Money,
    )
    loan_interest_rate: float = proto.Field(
        proto.FLOAT,
        number=3,
    )
    savings: "SavingsOverTime" = proto.Field(
        proto.MESSAGE,
        number=4,
        message="SavingsOverTime",
    )


class GetDataLayersRequest(proto.Message):
    r"""Request message for ``Solar.GetDataLayers``.

    Attributes:
        location (google.type.latlng_pb2.LatLng):
            Required. The longitude and latitude for the
            center of the region to get data for.
        radius_meters (float):
            Required. The radius, in meters, defining the region
            surrounding that centre point for which data should be
            returned. The limitations on this value are:

            -  Any value up to 100m can always be specified.
            -  Values over 100m can be specified, as long as
               ``radius_meters`` <= ``pixel_size_meters * 1000``.
            -  However, for values over 175m, the ``DataLayerView`` in
               the request must not include monthly flux or hourly
               shade.
        view (google.maps.solar_v1.types.DataLayerView):
            Optional. The desired subset of the data to
            return.
        required_quality (google.maps.solar_v1.types.ImageryQuality):
            Optional. The minimum quality level allowed
            in the results. No result with lower quality
            than this will be returned. Not specifying this
            is equivalent to restricting to HIGH quality
            only.
        pixel_size_meters (float):
            Optional. The minimum scale, in meters per pixel, of the
            data to return. Values of 0.1 (the default, if this field is
            not set explicitly), 0.25, 0.5, and 1.0 are supported.
            Imagery components whose normal resolution is less than
            ``pixel_size_meters`` will be returned at the resolution
            specified by ``pixel_size_meters``; imagery components whose
            normal resolution is equal to or greater than
            ``pixel_size_meters`` will be returned at that normal
            resolution.
        exact_quality_required (bool):
            Optional. Whether to require exact quality of the imagery.
            If set to false, the ``required_quality`` field is
            interpreted as the minimum required quality, such that HIGH
            quality imagery may be returned when ``required_quality`` is
            set to MEDIUM. If set to true, ``required_quality`` is
            interpreted as the exact required quality and only
            ``MEDIUM`` quality imagery is returned if
            ``required_quality`` is set to ``MEDIUM``.
    """

    location: latlng_pb2.LatLng = proto.Field(
        proto.MESSAGE,
        number=1,
        message=latlng_pb2.LatLng,
    )
    radius_meters: float = proto.Field(
        proto.FLOAT,
        number=2,
    )
    view: "DataLayerView" = proto.Field(
        proto.ENUM,
        number=3,
        enum="DataLayerView",
    )
    required_quality: "ImageryQuality" = proto.Field(
        proto.ENUM,
        number=5,
        enum="ImageryQuality",
    )
    pixel_size_meters: float = proto.Field(
        proto.FLOAT,
        number=6,
    )
    exact_quality_required: bool = proto.Field(
        proto.BOOL,
        number=7,
    )


class DataLayers(proto.Message):
    r"""Information about the solar potential of a region. The actual data
    are contained in a number of GeoTIFF files covering the requested
    region, for which this message contains URLs: Each string in the
    ``DataLayers`` message contains a URL from which the corresponding
    GeoTIFF can be fetched. These URLs are valid for a few hours after
    they've been generated. Most of the GeoTIFF files are at a
    resolution of 0.1m/pixel, but the monthly flux file is at
    0.5m/pixel, and the hourly shade files are at 1m/pixel. If a
    ``pixel_size_meters`` value was specified in the
    ``GetDataLayersRequest``, then the minimum resolution in the GeoTIFF
    files will be that value.

    Attributes:
        imagery_date (google.type.date_pb2.Date):
            When the source imagery (from which all the
            other data are derived) in this region was
            taken. It is necessarily somewhat approximate,
            as the images may have been taken over more than
            one day.
        imagery_processed_date (google.type.date_pb2.Date):
            When processing was completed on this
            imagery.
        dsm_url (str):
            The URL for an image of the DSM (Digital
            Surface Model) of the region. Values are in
            meters above EGM96 geoid (i.e., sea level).
            Invalid locations (where we don't have data) are
            stored as -9999.
        rgb_url (str):
            The URL for an image of RGB data (aerial
            photo) of the region.
        mask_url (str):
            The URL for the building mask image: one bit
            per pixel saying whether that pixel is
            considered to be part of a rooftop or not.
        annual_flux_url (str):
            The URL for the annual flux map (annual sunlight on roofs)
            of the region. Values are kWh/kW/year. This is *unmasked
            flux*: flux is computed for every location, not just
            building rooftops. Invalid locations are stored as -9999:
            locations outside our coverage area will be invalid, and a
            few locations inside the coverage area, where we were unable
            to calculate flux, will also be invalid.
        monthly_flux_url (str):
            The URL for the monthly flux map (sunlight on
            roofs, broken down by month) of the region.
            Values are kWh/kW/year. The GeoTIFF pointed to
            by this URL will contain twelve bands,
            corresponding to January...December, in order.
        hourly_shade_urls (MutableSequence[str]):
            Twelve URLs for hourly shade, corresponding to
            January...December, in order. Each GeoTIFF will contain 24
            bands, corresponding to the 24 hours of the day. Each pixel
            is a 32 bit integer, corresponding to the (up to) 31 days of
            that month; a 1 bit means that the corresponding location is
            able to see the sun at that day, of that hour, of that
            month. Invalid locations are stored as -9999 (since this is
            negative, it has bit 31 set, and no valid value could have
            bit 31 set as that would correspond to the 32nd day of the
            month).

            An example may be useful. If you want to know whether a
            point (at pixel location (x, y)) saw sun at 4pm on the 22nd
            of June you would:

            1. fetch the sixth URL in this list (corresponding to June).
            2. look up the 17th channel (corresponding to 4pm).
            3. read the 32-bit value at (x, y).
            4. read bit 21 of the value (corresponding to the 22nd of
               the month).
            5. if that bit is a 1, then that spot saw the sun at 4pm 22
               June.

            More formally: Given ``month`` (1-12), ``day`` (1...month
            max; February has 28 days) and ``hour`` (0-23), the
            shade/sun for that month/day/hour at a position ``(x, y)``
            is the bit

            ::

               (hourly_shade[month - 1])(x, y)[hour] & (1 << (day - 1))

            where ``(x, y)`` is spatial indexing, ``[month - 1]`` refers
            to fetching the ``month - 1``\ st URL (indexing from zero),
            ``[hour]`` is indexing into the channels, and a final
            non-zero result means "sunny". There are no leap days, and
            DST doesn't exist (all days are 24 hours long; noon is
            always "standard time" noon).
        imagery_quality (google.maps.solar_v1.types.ImageryQuality):
            The quality of the result's imagery.
    """

    imagery_date: date_pb2.Date = proto.Field(
        proto.MESSAGE,
        number=1,
        message=date_pb2.Date,
    )
    imagery_processed_date: date_pb2.Date = proto.Field(
        proto.MESSAGE,
        number=2,
        message=date_pb2.Date,
    )
    dsm_url: str = proto.Field(
        proto.STRING,
        number=3,
    )
    rgb_url: str = proto.Field(
        proto.STRING,
        number=4,
    )
    mask_url: str = proto.Field(
        proto.STRING,
        number=5,
    )
    annual_flux_url: str = proto.Field(
        proto.STRING,
        number=6,
    )
    monthly_flux_url: str = proto.Field(
        proto.STRING,
        number=7,
    )
    hourly_shade_urls: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=8,
    )
    imagery_quality: "ImageryQuality" = proto.Field(
        proto.ENUM,
        number=9,
        enum="ImageryQuality",
    )


class GetGeoTiffRequest(proto.Message):
    r"""Request message for ``Solar.GetGeoTiff``.

    Attributes:
        id (str):
            Required. The ID of the asset being
            requested.
    """

    id: str = proto.Field(
        proto.STRING,
        number=1,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import dataclasses
import json  # type: ignore
import logging
from typing import Any, Callable, Dict, List, Optional, Sequence, Tuple, Union
import warnings

from google.api import httpbody_pb2  # type: ignore
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1, rest_helpers, rest_streaming
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.transport.requests import AuthorizedSession  # type: ignore
from google.protobuf import json_format
from requests import __version__ as requests_version

from google.maps.solar_v1.types import solar_service

from .base import DEFAULT_CLIENT_INFO as BASE_DEFAULT_CLIENT_INFO
from .rest_base import _BaseSolarRestTransport

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object, None]  # type: ignore

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = logging.getLogger(__name__)

DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=BASE_DEFAULT_CLIENT_INFO.gapic_version,
    grpc_version=None,
    rest_version=f"requests@{requests_version}",
)


class SolarRestInterceptor:
    """Interceptor for Solar.

    Interceptors are used to manipulate requests, request metadata, and responses
    in arbitrary ways.
    Example use cases include:
    * Logging
    * Verifying requests according to service or custom semantics
    * Stripping extraneous information from responses

    These use cases and more can be enabled by injecting an
    instance of a custom subclass when constructing the SolarRestTransport.

    .. code-block:: python
        class MyCustomSolarInterceptor(SolarRestInterceptor):
            def pre_find_closest_building_insights(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_find_closest_building_insights(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_get_data_layers(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_get_data_layers(self, response):
                logging.log(f"Received response: {response}")
                return response

            def pre_get_geo_tiff(self, request, metadata):
                logging.log(f"Received request: {request}")
                return request, metadata

            def post_get_geo_tiff(self, response):
                logging.log(f"Received response: {response}")
                return response

        transport = SolarRestTransport(interceptor=MyCustomSolarInterceptor())
        client = SolarClient(transport=transport)


    """

    def pre_find_closest_building_insights(
        self,
        request: solar_service.FindClosestBuildingInsightsRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[
        solar_service.FindClosestBuildingInsightsRequest,
        Sequence[Tuple[str, Union[str, bytes]]],
    ]:
        """Pre-rpc interceptor for find_closest_building_insights

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Solar server.
        """
        return request, metadata

    def post_find_closest_building_insights(
        self, response: solar_service.BuildingInsights
    ) -> solar_service.BuildingInsights:
        """Post-rpc interceptor for find_closest_building_insights

        Override in a subclass to manipulate the response
        after it is returned by the Solar server but before
        it is returned to user code.
        """
        return response

    def pre_get_data_layers(
        self,
        request: solar_service.GetDataLayersRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[
        solar_service.GetDataLayersRequest, Sequence[Tuple[str, Union[str, bytes]]]
    ]:
        """Pre-rpc interceptor for get_data_layers

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Solar server.
        """
        return request, metadata

    def post_get_data_layers(
        self, response: solar_service.DataLayers
    ) -> solar_service.DataLayers:
        """Post-rpc interceptor for get_data_layers

        Override in a subclass to manipulate the response
        after it is returned by the Solar server but before
        it is returned to user code.
        """
        return response

    def pre_get_geo_tiff(
        self,
        request: solar_service.GetGeoTiffRequest,
        metadata: Sequence[Tuple[str, Union[str, bytes]]],
    ) -> Tuple[
        solar_service.GetGeoTiffRequest, Sequence[Tuple[str, Union[str, bytes]]]
    ]:
        """Pre-rpc interceptor for get_geo_tiff

        Override in a subclass to manipulate the request or metadata
        before they are sent to the Solar server.
        """
        return request, metadata

    def post_get_geo_tiff(
        self, response: httpbody_pb2.HttpBody
    ) -> httpbody_pb2.HttpBody:
        """Post-rpc interceptor for get_geo_tiff

        Override in a subclass to manipulate the response
        after it is returned by the Solar server but before
        it is returned to user code.
        """
        return response


@dataclasses.dataclass
class SolarRestStub:
    _session: AuthorizedSession
    _host: str
    _interceptor: SolarRestInterceptor


class SolarRestTransport(_BaseSolarRestTransport):
    """REST backend synchronous transport for Solar.

    Service definition for the Solar API.

    This class defines the same methods as the primary client, so the
    primary client can load the underlying transport implementation
    and call it.

    It sends JSON representations of protocol buffers over HTTP/1.1
    """

    def __init__(
        self,
        *,
        host: str = "solar.googleapis.com",
        credentials: Optional[ga_credentials.Credentials] = None,
        credentials_file: Optional[str] = None,
        scopes: Optional[Sequence[str]] = None,
        client_cert_source_for_mtls: Optional[Callable[[], Tuple[bytes, bytes]]] = None,
        quota_project_id: Optional[str] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
        always_use_jwt_access: Optional[bool] = False,
        url_scheme: str = "https",
        interceptor: Optional[SolarRestInterceptor] = None,
        api_audience: Optional[str] = None,
    ) -> None:
        """Instantiate the transport.

        Args:
            host (Optional[str]):
                 The hostname to connect to (default: 'solar.googleapis.com').
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.

            credentials_file (Optional[str]): A file with credentials that can
                be loaded with :func:`google.auth.load_credentials_from_file`.
                This argument is ignored if ``channel`` is provided.
            scopes (Optional(Sequence[str])): A list of scopes. This argument is
                ignored if ``channel`` is provided.
            client_cert_source_for_mtls (Callable[[], Tuple[bytes, bytes]]): Client
                certificate to configure mutual TLS HTTP channel. It is ignored
                if ``channel`` is provided.
            quota_project_id (Optional[str]): An optional project to use for billing
                and quota.
            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you are developing
                your own client library.
            always_use_jwt_access (Optional[bool]): Whether self signed JWT should
                be used for service account credentials.
            url_scheme: the protocol scheme for the API endpoint.  Normally
                "https", but for testing or local servers,
                "http" can be specified.
        """
        # Run the base constructor
        # TODO(yon-mg): resolve other ctor params i.e. scopes, quota, etc.
        # TODO: When custom host (api_endpoint) is set, `scopes` must *also* be set on the
        # credentials object
        super().__init__(
            host=host,
            credentials=credentials,
            client_info=client_info,
            always_use_jwt_access=always_use_jwt_access,
            url_scheme=url_scheme,
            api_audience=api_audience,
        )
        self._session = AuthorizedSession(
            self._credentials, default_host=self.DEFAULT_HOST
        )
        if client_cert_source_for_mtls:
            self._session.configure_mtls_channel(client_cert_source_for_mtls)
        self._interceptor = interceptor or SolarRestInterceptor()
        self._prep_wrapped_messages(client_info)

    class _FindClosestBuildingInsights(
        _BaseSolarRestTransport._BaseFindClosestBuildingInsights, SolarRestStub
    ):
        def __hash__(self):
            return hash("SolarRestTransport.FindClosestBuildingInsights")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: solar_service.FindClosestBuildingInsightsRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> solar_service.BuildingInsights:
            r"""Call the find closest building
            insights method over HTTP.

                Args:
                    request (~.solar_service.FindClosestBuildingInsightsRequest):
                        The request object. Request message for
                    ``Solar.FindClosestBuildingInsights``.
                    retry (google.api_core.retry.Retry): Designation of what errors, if any,
                        should be retried.
                    timeout (float): The timeout for this request.
                    metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                        sent along with the request as metadata. Normally, each value must be of type `str`,
                        but for metadata keys ending with the suffix `-bin`, the corresponding values must
                        be of type `bytes`.

                Returns:
                    ~.solar_service.BuildingInsights:
                        Response message for
                    ``Solar.FindClosestBuildingInsights``. Information about
                    the location, dimensions, and solar potential of a
                    building.

            """

            http_options = (
                _BaseSolarRestTransport._BaseFindClosestBuildingInsights._get_http_options()
            )

            request, metadata = self._interceptor.pre_find_closest_building_insights(
                request, metadata
            )
            transcoded_request = _BaseSolarRestTransport._BaseFindClosestBuildingInsights._get_transcoded_request(
                http_options, request
            )

            # Jsonify the query params
            query_params = _BaseSolarRestTransport._BaseFindClosestBuildingInsights._get_query_params_json(
                transcoded_request
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.maps.solar_v1.SolarClient.FindClosestBuildingInsights",
                    extra={
                        "serviceName": "google.maps.solar.v1.Solar",
                        "rpcName": "FindClosestBuildingInsights",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = SolarRestTransport._FindClosestBuildingInsights._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = solar_service.BuildingInsights()
            pb_resp = solar_service.BuildingInsights.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_find_closest_building_insights(resp)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = solar_service.BuildingInsights.to_json(response)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.maps.solar_v1.SolarClient.find_closest_building_insights",
                    extra={
                        "serviceName": "google.maps.solar.v1.Solar",
                        "rpcName": "FindClosestBuildingInsights",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _GetDataLayers(_BaseSolarRestTransport._BaseGetDataLayers, SolarRestStub):
        def __hash__(self):
            return hash("SolarRestTransport.GetDataLayers")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: solar_service.GetDataLayersRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> solar_service.DataLayers:
            r"""Call the get data layers method over HTTP.

            Args:
                request (~.solar_service.GetDataLayersRequest):
                    The request object. Request message for ``Solar.GetDataLayers``.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.solar_service.DataLayers:
                    Information about the solar potential of a region. The
                actual data are contained in a number of GeoTIFF files
                covering the requested region, for which this message
                contains URLs: Each string in the ``DataLayers`` message
                contains a URL from which the corresponding GeoTIFF can
                be fetched. These URLs are valid for a few hours after
                they've been generated. Most of the GeoTIFF files are at
                a resolution of 0.1m/pixel, but the monthly flux file is
                at 0.5m/pixel, and the hourly shade files are at
                1m/pixel. If a ``pixel_size_meters`` value was specified
                in the ``GetDataLayersRequest``, then the minimum
                resolution in the GeoTIFF files will be that value.

            """

            http_options = (
                _BaseSolarRestTransport._BaseGetDataLayers._get_http_options()
            )

            request, metadata = self._interceptor.pre_get_data_layers(request, metadata)
            transcoded_request = (
                _BaseSolarRestTransport._BaseGetDataLayers._get_transcoded_request(
                    http_options, request
                )
            )

            # Jsonify the query params
            query_params = (
                _BaseSolarRestTransport._BaseGetDataLayers._get_query_params_json(
                    transcoded_request
                )
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = type(request).to_json(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.maps.solar_v1.SolarClient.GetDataLayers",
                    extra={
                        "serviceName": "google.maps.solar.v1.Solar",
                        "rpcName": "GetDataLayers",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = SolarRestTransport._GetDataLayers._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = solar_service.DataLayers()
            pb_resp = solar_service.DataLayers.pb(resp)

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_get_data_layers(resp)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = solar_service.DataLayers.to_json(response)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.maps.solar_v1.SolarClient.get_data_layers",
                    extra={
                        "serviceName": "google.maps.solar.v1.Solar",
                        "rpcName": "GetDataLayers",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    class _GetGeoTiff(_BaseSolarRestTransport._BaseGetGeoTiff, SolarRestStub):
        def __hash__(self):
            return hash("SolarRestTransport.GetGeoTiff")

        @staticmethod
        def _get_response(
            host,
            metadata,
            query_params,
            session,
            timeout,
            transcoded_request,
            body=None,
        ):
            uri = transcoded_request["uri"]
            method = transcoded_request["method"]
            headers = dict(metadata)
            headers["Content-Type"] = "application/json"
            response = getattr(session, method)(
                "{host}{uri}".format(host=host, uri=uri),
                timeout=timeout,
                headers=headers,
                params=rest_helpers.flatten_query_params(query_params, strict=True),
            )
            return response

        def __call__(
            self,
            request: solar_service.GetGeoTiffRequest,
            *,
            retry: OptionalRetry = gapic_v1.method.DEFAULT,
            timeout: Optional[float] = None,
            metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
        ) -> httpbody_pb2.HttpBody:
            r"""Call the get geo tiff method over HTTP.

            Args:
                request (~.solar_service.GetGeoTiffRequest):
                    The request object. Request message for ``Solar.GetGeoTiff``.
                retry (google.api_core.retry.Retry): Designation of what errors, if any,
                    should be retried.
                timeout (float): The timeout for this request.
                metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                    sent along with the request as metadata. Normally, each value must be of type `str`,
                    but for metadata keys ending with the suffix `-bin`, the corresponding values must
                    be of type `bytes`.

            Returns:
                ~.httpbody_pb2.HttpBody:
                    Message that represents an arbitrary HTTP body. It
                should only be used for payload formats that can't be
                represented as JSON, such as raw binary or an HTML page.

                This message can be used both in streaming and
                non-streaming API methods in the request as well as the
                response.

                It can be used as a top-level request field, which is
                convenient if one wants to extract parameters from
                either the URL or HTTP template into the request fields
                and also want access to the raw HTTP body.

                Example:

                ::

                    message GetResourceRequest {
                      // A unique request id.
                      string request_id = 1;

                      // The raw HTTP body is bound to this field.
                      google.api.HttpBody http_body = 2;

                    }

                    service ResourceService {
                      rpc GetResource(GetResourceRequest)
                        returns (google.api.HttpBody);
                      rpc UpdateResource(google.api.HttpBody)
                        returns (google.protobuf.Empty);

                    }

                Example with streaming methods:

                ::

                    service CaldavService {
                      rpc GetCalendar(stream google.api.HttpBody)
                        returns (stream google.api.HttpBody);
                      rpc UpdateCalendar(stream google.api.HttpBody)
                        returns (stream google.api.HttpBody);

                    }

                Use of this type only changes how the request and
                response bodies are handled, all other features will
                continue to work unchanged.

            """

            http_options = _BaseSolarRestTransport._BaseGetGeoTiff._get_http_options()

            request, metadata = self._interceptor.pre_get_geo_tiff(request, metadata)
            transcoded_request = (
                _BaseSolarRestTransport._BaseGetGeoTiff._get_transcoded_request(
                    http_options, request
                )
            )

            # Jsonify the query params
            query_params = (
                _BaseSolarRestTransport._BaseGetGeoTiff._get_query_params_json(
                    transcoded_request
                )
            )

            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                request_url = "{host}{uri}".format(
                    host=self._host, uri=transcoded_request["uri"]
                )
                method = transcoded_request["method"]
                try:
                    request_payload = json_format.MessageToJson(request)
                except:
                    request_payload = None
                http_request = {
                    "payload": request_payload,
                    "requestMethod": method,
                    "requestUrl": request_url,
                    "headers": dict(metadata),
                }
                _LOGGER.debug(
                    f"Sending request for google.maps.solar_v1.SolarClient.GetGeoTiff",
                    extra={
                        "serviceName": "google.maps.solar.v1.Solar",
                        "rpcName": "GetGeoTiff",
                        "httpRequest": http_request,
                        "metadata": http_request["headers"],
                    },
                )

            # Send the request
            response = SolarRestTransport._GetGeoTiff._get_response(
                self._host,
                metadata,
                query_params,
                self._session,
                timeout,
                transcoded_request,
            )

            # In case of error, raise the appropriate core_exceptions.GoogleAPICallError exception
            # subclass.
            if response.status_code >= 400:
                raise core_exceptions.from_http_response(response)

            # Return the response
            resp = httpbody_pb2.HttpBody()
            pb_resp = resp

            json_format.Parse(response.content, pb_resp, ignore_unknown_fields=True)

            resp = self._interceptor.post_get_geo_tiff(resp)
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                logging.DEBUG
            ):  # pragma: NO COVER
                try:
                    response_payload = json_format.MessageToJson(resp)
                except:
                    response_payload = None
                http_response = {
                    "payload": response_payload,
                    "headers": dict(response.headers),
                    "status": response.status_code,
                }
                _LOGGER.debug(
                    "Received response for google.maps.solar_v1.SolarClient.get_geo_tiff",
                    extra={
                        "serviceName": "google.maps.solar.v1.Solar",
                        "rpcName": "GetGeoTiff",
                        "metadata": http_response["headers"],
                        "httpResponse": http_response,
                    },
                )
            return resp

    @property
    def find_closest_building_insights(
        self,
    ) -> Callable[
        [solar_service.FindClosestBuildingInsightsRequest],
        solar_service.BuildingInsights,
    ]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._FindClosestBuildingInsights(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def get_data_layers(
        self,
    ) -> Callable[[solar_service.GetDataLayersRequest], solar_service.DataLayers]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._GetDataLayers(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def get_geo_tiff(
        self,
    ) -> Callable[[solar_service.GetGeoTiffRequest], httpbody_pb2.HttpBody]:
        # The return type is fine, but mypy isn't sophisticated enough to determine what's going on here.
        # In C++ this would require a dynamic_cast
        return self._GetGeoTiff(self._session, self._host, self._interceptor)  # type: ignore

    @property
    def kind(self) -> str:
        return "rest"

    def close(self):
        self._session.close()


__all__ = ("SolarRestTransport",)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import logging as std_logging
import os
import re
from typing import (
    Callable,
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
    cast,
)
import warnings

from google.api_core import client_options as client_options_lib
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.auth.exceptions import MutualTLSChannelError  # type: ignore
from google.auth.transport import mtls  # type: ignore
from google.auth.transport.grpc import SslCredentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.maps.solar_v1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.Retry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.Retry, object, None]  # type: ignore

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)

from google.api import httpbody_pb2  # type: ignore
from google.protobuf import any_pb2  # type: ignore
from google.type import date_pb2  # type: ignore
from google.type import latlng_pb2  # type: ignore

from google.maps.solar_v1.types import solar_service

from .transports.base import DEFAULT_CLIENT_INFO, SolarTransport
from .transports.grpc import SolarGrpcTransport
from .transports.grpc_asyncio import SolarGrpcAsyncIOTransport
from .transports.rest import SolarRestTransport


class SolarClientMeta(type):
    """Metaclass for the Solar client.

    This provides class-level methods for building and retrieving
    support objects (e.g. transport) without polluting the client instance
    objects.
    """

    _transport_registry = OrderedDict()  # type: Dict[str, Type[SolarTransport]]
    _transport_registry["grpc"] = SolarGrpcTransport
    _transport_registry["grpc_asyncio"] = SolarGrpcAsyncIOTransport
    _transport_registry["rest"] = SolarRestTransport

    def get_transport_class(
        cls,
        label: Optional[str] = None,
    ) -> Type[SolarTransport]:
        """Returns an appropriate transport class.

        Args:
            label: The name of the desired transport. If none is
                provided, then the first transport in the registry is used.

        Returns:
            The transport class to use.
        """
        # If a specific transport is requested, return that one.
        if label:
            return cls._transport_registry[label]

        # No transport is requested; return the default (that is, the first one
        # in the dictionary).
        return next(iter(cls._transport_registry.values()))


class SolarClient(metaclass=SolarClientMeta):
    """Service definition for the Solar API."""

    @staticmethod
    def _get_default_mtls_endpoint(api_endpoint):
        """Converts api endpoint to mTLS endpoint.

        Convert "*.sandbox.googleapis.com" and "*.googleapis.com" to
        "*.mtls.sandbox.googleapis.com" and "*.mtls.googleapis.com" respectively.
        Args:
            api_endpoint (Optional[str]): the api endpoint to convert.
        Returns:
            str: converted mTLS api endpoint.
        """
        if not api_endpoint:
            return api_endpoint

        mtls_endpoint_re = re.compile(
            r"(?P<name>[^.]+)(?P<mtls>\.mtls)?(?P<sandbox>\.sandbox)?(?P<googledomain>\.googleapis\.com)?"
        )

        m = mtls_endpoint_re.match(api_endpoint)
        name, mtls, sandbox, googledomain = m.groups()
        if mtls or not googledomain:
            return api_endpoint

        if sandbox:
            return api_endpoint.replace(
                "sandbox.googleapis.com", "mtls.sandbox.googleapis.com"
            )

        return api_endpoint.replace(".googleapis.com", ".mtls.googleapis.com")

    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = "solar.googleapis.com"
    DEFAULT_MTLS_ENDPOINT = _get_default_mtls_endpoint.__func__(  # type: ignore
        DEFAULT_ENDPOINT
    )

    _DEFAULT_ENDPOINT_TEMPLATE = "solar.{UNIVERSE_DOMAIN}"
    _DEFAULT_UNIVERSE = "googleapis.com"

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SolarClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_info(info)
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SolarClient: The constructed client.
        """
        credentials = service_account.Credentials.from_service_account_file(filename)
        kwargs["credentials"] = credentials
        return cls(*args, **kwargs)

    from_service_account_json = from_service_account_file

    @property
    def transport(self) -> SolarTransport:
        """Returns the transport used by the client instance.

        Returns:
            SolarTransport: The transport used by the client
                instance.
        """
        return self._transport

    @staticmethod
    def common_billing_account_path(
        billing_account: str,
    ) -> str:
        """Returns a fully-qualified billing_account string."""
        return "billingAccounts/{billing_account}".format(
            billing_account=billing_account,
        )

    @staticmethod
    def parse_common_billing_account_path(path: str) -> Dict[str, str]:
        """Parse a billing_account path into its component segments."""
        m = re.match(r"^billingAccounts/(?P<billing_account>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_folder_path(
        folder: str,
    ) -> str:
        """Returns a fully-qualified folder string."""
        return "folders/{folder}".format(
            folder=folder,
        )

    @staticmethod
    def parse_common_folder_path(path: str) -> Dict[str, str]:
        """Parse a folder path into its component segments."""
        m = re.match(r"^folders/(?P<folder>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_organization_path(
        organization: str,
    ) -> str:
        """Returns a fully-qualified organization string."""
        return "organizations/{organization}".format(
            organization=organization,
        )

    @staticmethod
    def parse_common_organization_path(path: str) -> Dict[str, str]:
        """Parse a organization path into its component segments."""
        m = re.match(r"^organizations/(?P<organization>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_project_path(
        project: str,
    ) -> str:
        """Returns a fully-qualified project string."""
        return "projects/{project}".format(
            project=project,
        )

    @staticmethod
    def parse_common_project_path(path: str) -> Dict[str, str]:
        """Parse a project path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)$", path)
        return m.groupdict() if m else {}

    @staticmethod
    def common_location_path(
        project: str,
        location: str,
    ) -> str:
        """Returns a fully-qualified location string."""
        return "projects/{project}/locations/{location}".format(
            project=project,
            location=location,
        )

    @staticmethod
    def parse_common_location_path(path: str) -> Dict[str, str]:
        """Parse a location path into its component segments."""
        m = re.match(r"^projects/(?P<project>.+?)/locations/(?P<location>.+?)$", path)
        return m.groupdict() if m else {}

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[client_options_lib.ClientOptions] = None
    ):
        """Deprecated. Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """

        warnings.warn(
            "get_mtls_endpoint_and_cert_source is deprecated. Use the api_endpoint property instead.",
            DeprecationWarning,
        )
        if client_options is None:
            client_options = client_options_lib.ClientOptions()
        use_client_cert = os.getenv("GOOGLE_API_USE_CLIENT_CERTIFICATE", "false")
        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto")
        if use_client_cert not in ("true", "false"):
            raise ValueError(
                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
            )
        if use_mtls_endpoint not in ("auto", "never", "always"):
            raise MutualTLSChannelError(
                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
            )

        # Figure out the client cert source to use.
        client_cert_source = None
        if use_client_cert == "true":
            if client_options.client_cert_source:
                client_cert_source = client_options.client_cert_source
            elif mtls.has_default_client_cert_source():
                client_cert_source = mtls.default_client_cert_source()

        # Figure out which api endpoint to use.
        if client_options.api_endpoint is not None:
            api_endpoint = client_options.api_endpoint
        elif use_mtls_endpoint == "always" or (
            use_mtls_endpoint == "auto" and client_cert_source
        ):
            api_endpoint = cls.DEFAULT_MTLS_ENDPOINT
        else:
            api_endpoint = cls.DEFAULT_ENDPOINT

        return api_endpoint, client_cert_source

    @staticmethod
    def _read_environment_variables():
        """Returns the environment variables used by the client.

        Returns:
            Tuple[bool, str, str]: returns the GOOGLE_API_USE_CLIENT_CERTIFICATE,
            GOOGLE_API_USE_MTLS_ENDPOINT, and GOOGLE_CLOUD_UNIVERSE_DOMAIN environment variables.

        Raises:
            ValueError: If GOOGLE_API_USE_CLIENT_CERTIFICATE is not
                any of ["true", "false"].
            google.auth.exceptions.MutualTLSChannelError: If GOOGLE_API_USE_MTLS_ENDPOINT
                is not any of ["auto", "never", "always"].
        """
        use_client_cert = os.getenv(
            "GOOGLE_API_USE_CLIENT_CERTIFICATE", "false"
        ).lower()
        use_mtls_endpoint = os.getenv("GOOGLE_API_USE_MTLS_ENDPOINT", "auto").lower()
        universe_domain_env = os.getenv("GOOGLE_CLOUD_UNIVERSE_DOMAIN")
        if use_client_cert not in ("true", "false"):
            raise ValueError(
                "Environment variable `GOOGLE_API_USE_CLIENT_CERTIFICATE` must be either `true` or `false`"
            )
        if use_mtls_endpoint not in ("auto", "never", "always"):
            raise MutualTLSChannelError(
                "Environment variable `GOOGLE_API_USE_MTLS_ENDPOINT` must be `never`, `auto` or `always`"
            )
        return use_client_cert == "true", use_mtls_endpoint, universe_domain_env

    @staticmethod
    def _get_client_cert_source(provided_cert_source, use_cert_flag):
        """Return the client cert source to be used by the client.

        Args:
            provided_cert_source (bytes): The client certificate source provided.
            use_cert_flag (bool): A flag indicating whether to use the client certificate.

        Returns:
            bytes or None: The client cert source to be used by the client.
        """
        client_cert_source = None
        if use_cert_flag:
            if provided_cert_source:
                client_cert_source = provided_cert_source
            elif mtls.has_default_client_cert_source():
                client_cert_source = mtls.default_client_cert_source()
        return client_cert_source

    @staticmethod
    def _get_api_endpoint(
        api_override, client_cert_source, universe_domain, use_mtls_endpoint
    ):
        """Return the API endpoint used by the client.

        Args:
            api_override (str): The API endpoint override. If specified, this is always
                the return value of this function and the other arguments are not used.
            client_cert_source (bytes): The client certificate source used by the client.
            universe_domain (str): The universe domain used by the client.
            use_mtls_endpoint (str): How to use the mTLS endpoint, which depends also on the other parameters.
                Possible values are "always", "auto", or "never".

        Returns:
            str: The API endpoint to be used by the client.
        """
        if api_override is not None:
            api_endpoint = api_override
        elif use_mtls_endpoint == "always" or (
            use_mtls_endpoint == "auto" and client_cert_source
        ):
            _default_universe = SolarClient._DEFAULT_UNIVERSE
            if universe_domain != _default_universe:
                raise MutualTLSChannelError(
                    f"mTLS is not supported in any universe other than {_default_universe}."
                )
            api_endpoint = SolarClient.DEFAULT_MTLS_ENDPOINT
        else:
            api_endpoint = SolarClient._DEFAULT_ENDPOINT_TEMPLATE.format(
                UNIVERSE_DOMAIN=universe_domain
            )
        return api_endpoint

    @staticmethod
    def _get_universe_domain(
        client_universe_domain: Optional[str], universe_domain_env: Optional[str]
    ) -> str:
        """Return the universe domain used by the client.

        Args:
            client_universe_domain (Optional[str]): The universe domain configured via the client options.
            universe_domain_env (Optional[str]): The universe domain configured via the "GOOGLE_CLOUD_UNIVERSE_DOMAIN" environment variable.

        Returns:
            str: The universe domain to be used by the client.

        Raises:
            ValueError: If the universe domain is an empty string.
        """
        universe_domain = SolarClient._DEFAULT_UNIVERSE
        if client_universe_domain is not None:
            universe_domain = client_universe_domain
        elif universe_domain_env is not None:
            universe_domain = universe_domain_env
        if len(universe_domain.strip()) == 0:
            raise ValueError("Universe Domain cannot be an empty string.")
        return universe_domain

    def _validate_universe_domain(self):
        """Validates client's and credentials' universe domains are consistent.

        Returns:
            bool: True iff the configured universe domain is valid.

        Raises:
            ValueError: If the configured universe domain is not valid.
        """

        # NOTE (b/349488459): universe validation is disabled until further notice.
        return True

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used by the client instance.
        """
        return self._universe_domain

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[str, SolarTransport, Callable[..., SolarTransport]]
        ] = None,
        client_options: Optional[Union[client_options_lib.ClientOptions, dict]] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the solar client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,SolarTransport,Callable[..., SolarTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the SolarTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that the ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client_options = client_options
        if isinstance(self._client_options, dict):
            self._client_options = client_options_lib.from_dict(self._client_options)
        if self._client_options is None:
            self._client_options = client_options_lib.ClientOptions()
        self._client_options = cast(
            client_options_lib.ClientOptions, self._client_options
        )

        universe_domain_opt = getattr(self._client_options, "universe_domain", None)

        (
            self._use_client_cert,
            self._use_mtls_endpoint,
            self._universe_domain_env,
        ) = SolarClient._read_environment_variables()
        self._client_cert_source = SolarClient._get_client_cert_source(
            self._client_options.client_cert_source, self._use_client_cert
        )
        self._universe_domain = SolarClient._get_universe_domain(
            universe_domain_opt, self._universe_domain_env
        )
        self._api_endpoint = None  # updated below, depending on `transport`

        # Initialize the universe domain validation.
        self._is_universe_domain_valid = False

        if CLIENT_LOGGING_SUPPORTED:  # pragma: NO COVER
            # Setup logging.
            client_logging.initialize_logging()

        api_key_value = getattr(self._client_options, "api_key", None)
        if api_key_value and credentials:
            raise ValueError(
                "client_options.api_key and credentials are mutually exclusive"
            )

        # Save or instantiate the transport.
        # Ordinarily, we provide the transport, but allowing a custom transport
        # instance provides an extensibility point for unusual situations.
        transport_provided = isinstance(transport, SolarTransport)
        if transport_provided:
            # transport is a SolarTransport instance.
            if credentials or self._client_options.credentials_file or api_key_value:
                raise ValueError(
                    "When providing a transport instance, "
                    "provide its credentials directly."
                )
            if self._client_options.scopes:
                raise ValueError(
                    "When providing a transport instance, provide its scopes "
                    "directly."
                )
            self._transport = cast(SolarTransport, transport)
            self._api_endpoint = self._transport.host

        self._api_endpoint = self._api_endpoint or SolarClient._get_api_endpoint(
            self._client_options.api_endpoint,
            self._client_cert_source,
            self._universe_domain,
            self._use_mtls_endpoint,
        )

        if not transport_provided:
            import google.auth._default  # type: ignore

            if api_key_value and hasattr(
                google.auth._default, "get_api_key_credentials"
            ):
                credentials = google.auth._default.get_api_key_credentials(
                    api_key_value
                )

            transport_init: Union[
                Type[SolarTransport], Callable[..., SolarTransport]
            ] = (
                SolarClient.get_transport_class(transport)
                if isinstance(transport, str) or transport is None
                else cast(Callable[..., SolarTransport], transport)
            )
            # initialize with the provided callable or the passed in class
            self._transport = transport_init(
                credentials=credentials,
                credentials_file=self._client_options.credentials_file,
                host=self._api_endpoint,
                scopes=self._client_options.scopes,
                client_cert_source_for_mtls=self._client_cert_source,
                quota_project_id=self._client_options.quota_project_id,
                client_info=client_info,
                always_use_jwt_access=True,
                api_audience=self._client_options.api_audience,
            )

        if "async" not in str(self._transport):
            if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
                std_logging.DEBUG
            ):  # pragma: NO COVER
                _LOGGER.debug(
                    "Created client `google.maps.solar_v1.SolarClient`.",
                    extra={
                        "serviceName": "google.maps.solar.v1.Solar",
                        "universeDomain": getattr(
                            self._transport._credentials, "universe_domain", ""
                        ),
                        "credentialsType": f"{type(self._transport._credentials).__module__}.{type(self._transport._credentials).__qualname__}",
                        "credentialsInfo": getattr(
                            self.transport._credentials, "get_cred_info", lambda: None
                        )(),
                    }
                    if hasattr(self._transport, "_credentials")
                    else {
                        "serviceName": "google.maps.solar.v1.Solar",
                        "credentialsType": None,
                    },
                )

    def find_closest_building_insights(
        self,
        request: Optional[
            Union[solar_service.FindClosestBuildingInsightsRequest, dict]
        ] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> solar_service.BuildingInsights:
        r"""Locates the closest building to a query point. Returns an error
        with code ``NOT_FOUND`` if there are no buildings within
        approximately 50m of the query point.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.maps import solar_v1

            def sample_find_closest_building_insights():
                # Create a client
                client = solar_v1.SolarClient()

                # Initialize request argument(s)
                request = solar_v1.FindClosestBuildingInsightsRequest(
                )

                # Make the request
                response = client.find_closest_building_insights(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.maps.solar_v1.types.FindClosestBuildingInsightsRequest, dict]):
                The request object. Request message for
                ``Solar.FindClosestBuildingInsights``.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.maps.solar_v1.types.BuildingInsights:
                Response message for Solar.FindClosestBuildingInsights.
                   Information about the location, dimensions, and solar
                   potential of a building.

        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, solar_service.FindClosestBuildingInsightsRequest):
            request = solar_service.FindClosestBuildingInsightsRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[
            self._transport.find_closest_building_insights
        ]

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def get_data_layers(
        self,
        request: Optional[Union[solar_service.GetDataLayersRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> solar_service.DataLayers:
        r"""Gets solar information for a region surrounding a location.
        Returns an error with code ``NOT_FOUND`` if the location is
        outside the coverage area.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.maps import solar_v1

            def sample_get_data_layers():
                # Create a client
                client = solar_v1.SolarClient()

                # Initialize request argument(s)
                request = solar_v1.GetDataLayersRequest(
                    radius_meters=0.1399,
                )

                # Make the request
                response = client.get_data_layers(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.maps.solar_v1.types.GetDataLayersRequest, dict]):
                The request object. Request message for ``Solar.GetDataLayers``.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.maps.solar_v1.types.DataLayers:
                Information about the solar potential of a region. The actual data
                   are contained in a number of GeoTIFF files covering
                   the requested region, for which this message contains
                   URLs: Each string in the DataLayers message contains
                   a URL from which the corresponding GeoTIFF can be
                   fetched. These URLs are valid for a few hours after
                   they've been generated. Most of the GeoTIFF files are
                   at a resolution of 0.1m/pixel, but the monthly flux
                   file is at 0.5m/pixel, and the hourly shade files are
                   at 1m/pixel. If a pixel_size_meters value was
                   specified in the GetDataLayersRequest, then the
                   minimum resolution in the GeoTIFF files will be that
                   value.

        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, solar_service.GetDataLayersRequest):
            request = solar_service.GetDataLayersRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.get_data_layers]

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def get_geo_tiff(
        self,
        request: Optional[Union[solar_service.GetGeoTiffRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> httpbody_pb2.HttpBody:
        r"""Returns an image by its ID.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.maps import solar_v1

            def sample_get_geo_tiff():
                # Create a client
                client = solar_v1.SolarClient()

                # Initialize request argument(s)
                request = solar_v1.GetGeoTiffRequest(
                    id="id_value",
                )

                # Make the request
                response = client.get_geo_tiff(request=request)

                # Handle the response
                print(response)

        Args:
            request (Union[google.maps.solar_v1.types.GetGeoTiffRequest, dict]):
                The request object. Request message for ``Solar.GetGeoTiff``.
            retry (google.api_core.retry.Retry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api.httpbody_pb2.HttpBody:
                Message that represents an arbitrary HTTP body. It should only be used for
                   payload formats that can't be represented as JSON,
                   such as raw binary or an HTML page.

                   This message can be used both in streaming and
                   non-streaming API methods in the request as well as
                   the response.

                   It can be used as a top-level request field, which is
                   convenient if one wants to extract parameters from
                   either the URL or HTTP template into the request
                   fields and also want access to the raw HTTP body.

                   Example:

                      message GetResourceRequest {
                         // A unique request id. string request_id = 1;

                         // The raw HTTP body is bound to this field.
                         google.api.HttpBody http_body = 2;

                      }

                      service ResourceService {
                         rpc GetResource(GetResourceRequest)
                            returns (google.api.HttpBody);

                         rpc UpdateResource(google.api.HttpBody)
                            returns (google.protobuf.Empty);

                      }

                   Example with streaming methods:

                      service CaldavService {
                         rpc GetCalendar(stream google.api.HttpBody)
                            returns (stream google.api.HttpBody);

                         rpc UpdateCalendar(stream google.api.HttpBody)
                            returns (stream google.api.HttpBody);

                      }

                   Use of this type only changes how the request and
                   response bodies are handled, all other features will
                   continue to work unchanged.

        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, solar_service.GetGeoTiffRequest):
            request = solar_service.GetGeoTiffRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._transport._wrapped_methods[self._transport.get_geo_tiff]

        # Validate the universe domain.
        self._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def __enter__(self) -> "SolarClient":
        return self

    def __exit__(self, type, value, traceback):
        """Releases underlying transport's resources.

        .. warning::
            ONLY use as a context manager if the transport is NOT shared
            with other clients! Exiting the with block will CLOSE the transport
            and may cause errors in other clients!
        """
        self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("SolarClient",)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from .solar_service import (
    BuildingInsights,
    CashPurchaseSavings,
    DataLayers,
    DataLayerView,
    FinancedPurchaseSavings,
    FinancialAnalysis,
    FinancialDetails,
    FindClosestBuildingInsightsRequest,
    GetDataLayersRequest,
    GetGeoTiffRequest,
    ImageryQuality,
    LatLngBox,
    LeasingSavings,
    RoofSegmentSizeAndSunshineStats,
    RoofSegmentSummary,
    SavingsOverTime,
    SizeAndSunshineStats,
    SolarPanel,
    SolarPanelConfig,
    SolarPanelOrientation,
    SolarPotential,
)

__all__ = (
    "BuildingInsights",
    "CashPurchaseSavings",
    "DataLayers",
    "FinancedPurchaseSavings",
    "FinancialAnalysis",
    "FinancialDetails",
    "FindClosestBuildingInsightsRequest",
    "GetDataLayersRequest",
    "GetGeoTiffRequest",
    "LatLngBox",
    "LeasingSavings",
    "RoofSegmentSizeAndSunshineStats",
    "RoofSegmentSummary",
    "SavingsOverTime",
    "SizeAndSunshineStats",
    "SolarPanel",
    "SolarPanelConfig",
    "SolarPotential",
    "DataLayerView",
    "ImageryQuality",
    "SolarPanelOrientation",
)



# -*- coding: utf-8 -*-
# Copyright 2024 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from collections import OrderedDict
import logging as std_logging
import re
from typing import (
    Callable,
    Dict,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.api_core.client_options import ClientOptions
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore

from google.maps.solar_v1 import gapic_version as package_version

try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from google.api import httpbody_pb2  # type: ignore
from google.protobuf import any_pb2  # type: ignore
from google.type import date_pb2  # type: ignore
from google.type import latlng_pb2  # type: ignore

from google.maps.solar_v1.types import solar_service

from .client import SolarClient
from .transports.base import DEFAULT_CLIENT_INFO, SolarTransport
from .transports.grpc_asyncio import SolarGrpcAsyncIOTransport

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)


class SolarAsyncClient:
    """Service definition for the Solar API."""

    _client: SolarClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = SolarClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = SolarClient.DEFAULT_MTLS_ENDPOINT
    _DEFAULT_ENDPOINT_TEMPLATE = SolarClient._DEFAULT_ENDPOINT_TEMPLATE
    _DEFAULT_UNIVERSE = SolarClient._DEFAULT_UNIVERSE

    common_billing_account_path = staticmethod(SolarClient.common_billing_account_path)
    parse_common_billing_account_path = staticmethod(
        SolarClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(SolarClient.common_folder_path)
    parse_common_folder_path = staticmethod(SolarClient.parse_common_folder_path)
    common_organization_path = staticmethod(SolarClient.common_organization_path)
    parse_common_organization_path = staticmethod(
        SolarClient.parse_common_organization_path
    )
    common_project_path = staticmethod(SolarClient.common_project_path)
    parse_common_project_path = staticmethod(SolarClient.parse_common_project_path)
    common_location_path = staticmethod(SolarClient.common_location_path)
    parse_common_location_path = staticmethod(SolarClient.parse_common_location_path)

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SolarAsyncClient: The constructed client.
        """
        return SolarClient.from_service_account_info.__func__(SolarAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            SolarAsyncClient: The constructed client.
        """
        return SolarClient.from_service_account_file.__func__(SolarAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return SolarClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> SolarTransport:
        """Returns the transport used by the client instance.

        Returns:
            SolarTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = SolarClient.get_transport_class

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[str, SolarTransport, Callable[..., SolarTransport]]
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the solar async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,SolarTransport,Callable[..., SolarTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the SolarTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = SolarClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

        if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
            std_logging.DEBUG
        ):  # pragma: NO COVER
            _LOGGER.debug(
                "Created client `google.maps.solar_v1.SolarAsyncClient`.",
                extra={
                    "serviceName": "google.maps.solar.v1.Solar",
                    "universeDomain": getattr(
                        self._client._transport._credentials, "universe_domain", ""
                    ),
                    "credentialsType": f"{type(self._client._transport._credentials).__module__}.{type(self._client._transport._credentials).__qualname__}",
                    "credentialsInfo": getattr(
                        self.transport._credentials, "get_cred_info", lambda: None
                    )(),
                }
                if hasattr(self._client._transport, "_credentials")
                else {
                    "serviceName": "google.maps.solar.v1.Solar",
                    "credentialsType": None,
                },
            )

    async def find_closest_building_insights(
        self,
        request: Optional[
            Union[solar_service.FindClosestBuildingInsightsRequest, dict]
        ] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> solar_service.BuildingInsights:
        r"""Locates the closest building to a query point. Returns an error
        with code ``NOT_FOUND`` if there are no buildings within
        approximately 50m of the query point.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.maps import solar_v1

            async def sample_find_closest_building_insights():
                # Create a client
                client = solar_v1.SolarAsyncClient()

                # Initialize request argument(s)
                request = solar_v1.FindClosestBuildingInsightsRequest(
                )

                # Make the request
                response = await client.find_closest_building_insights(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.maps.solar_v1.types.FindClosestBuildingInsightsRequest, dict]]):
                The request object. Request message for
                ``Solar.FindClosestBuildingInsights``.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.maps.solar_v1.types.BuildingInsights:
                Response message for Solar.FindClosestBuildingInsights.
                   Information about the location, dimensions, and solar
                   potential of a building.

        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, solar_service.FindClosestBuildingInsightsRequest):
            request = solar_service.FindClosestBuildingInsightsRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.find_closest_building_insights
        ]

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_data_layers(
        self,
        request: Optional[Union[solar_service.GetDataLayersRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> solar_service.DataLayers:
        r"""Gets solar information for a region surrounding a location.
        Returns an error with code ``NOT_FOUND`` if the location is
        outside the coverage area.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.maps import solar_v1

            async def sample_get_data_layers():
                # Create a client
                client = solar_v1.SolarAsyncClient()

                # Initialize request argument(s)
                request = solar_v1.GetDataLayersRequest(
                    radius_meters=0.1399,
                )

                # Make the request
                response = await client.get_data_layers(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.maps.solar_v1.types.GetDataLayersRequest, dict]]):
                The request object. Request message for ``Solar.GetDataLayers``.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.maps.solar_v1.types.DataLayers:
                Information about the solar potential of a region. The actual data
                   are contained in a number of GeoTIFF files covering
                   the requested region, for which this message contains
                   URLs: Each string in the DataLayers message contains
                   a URL from which the corresponding GeoTIFF can be
                   fetched. These URLs are valid for a few hours after
                   they've been generated. Most of the GeoTIFF files are
                   at a resolution of 0.1m/pixel, but the monthly flux
                   file is at 0.5m/pixel, and the hourly shade files are
                   at 1m/pixel. If a pixel_size_meters value was
                   specified in the GetDataLayersRequest, then the
                   minimum resolution in the GeoTIFF files will be that
                   value.

        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, solar_service.GetDataLayersRequest):
            request = solar_service.GetDataLayersRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.get_data_layers
        ]

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def get_geo_tiff(
        self,
        request: Optional[Union[solar_service.GetGeoTiffRequest, dict]] = None,
        *,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> httpbody_pb2.HttpBody:
        r"""Returns an image by its ID.

        .. code-block:: python

            # This snippet has been automatically generated and should be regarded as a
            # code template only.
            # It will require modifications to work:
            # - It may require correct/in-range values for request initialization.
            # - It may require specifying regional endpoints when creating the service
            #   client as shown in:
            #   https://googleapis.dev/python/google-api-core/latest/client_options.html
            from google.maps import solar_v1

            async def sample_get_geo_tiff():
                # Create a client
                client = solar_v1.SolarAsyncClient()

                # Initialize request argument(s)
                request = solar_v1.GetGeoTiffRequest(
                    id="id_value",
                )

                # Make the request
                response = await client.get_geo_tiff(request=request)

                # Handle the response
                print(response)

        Args:
            request (Optional[Union[google.maps.solar_v1.types.GetGeoTiffRequest, dict]]):
                The request object. Request message for ``Solar.GetGeoTiff``.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.api.httpbody_pb2.HttpBody:
                Message that represents an arbitrary HTTP body. It should only be used for
                   payload formats that can't be represented as JSON,
                   such as raw binary or an HTML page.

                   This message can be used both in streaming and
                   non-streaming API methods in the request as well as
                   the response.

                   It can be used as a top-level request field, which is
                   convenient if one wants to extract parameters from
                   either the URL or HTTP template into the request
                   fields and also want access to the raw HTTP body.

                   Example:

                      message GetResourceRequest {
                         // A unique request id. string request_id = 1;

                         // The raw HTTP body is bound to this field.
                         google.api.HttpBody http_body = 2;

                      }

                      service ResourceService {
                         rpc GetResource(GetResourceRequest)
                            returns (google.api.HttpBody);

                         rpc UpdateResource(google.api.HttpBody)
                            returns (google.protobuf.Empty);

                      }

                   Example with streaming methods:

                      service CaldavService {
                         rpc GetCalendar(stream google.api.HttpBody)
                            returns (stream google.api.HttpBody);

                         rpc UpdateCalendar(stream google.api.HttpBody)
                            returns (stream google.api.HttpBody);

                      }

                   Use of this type only changes how the request and
                   response bodies are handled, all other features will
                   continue to work unchanged.

        """
        # Create or coerce a protobuf request object.
        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, solar_service.GetGeoTiffRequest):
            request = solar_service.GetGeoTiffRequest(request)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.get_geo_tiff
        ]

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "SolarAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)


__all__ = ("SolarAsyncClient",)
